schema {
  query: Query
  mutation: Mutation
  subscription: Subscription
}
scalar BigInt
"An instantaneous point on the time-line represented by a standard date time string"
scalar Instant
scalar SessionId
scalar Unit

union TransactionSyncEvent = ProgressUpdate | TransactionAdded

union WalletSyncEvent = ProgressUpdate | ViewingUpdate

union ZswapChainStateUpdate = MerkleTreeCollapsedUpdate | RelevantTransaction

input BlockOffsetInput {
  hash: String
  height: BigInt
}

input TransactionOffsetInput {
  hash: String
  identifier: String
}

interface ContractCallOrDeploy {
  address: String!
  state: String!
  transaction: Transaction! @cost(weight: 10.0)
  zswapChainState: String!
}

type Block {
  parent: Block @cost(weight: 10.0)
  hash: String!
  height: BigInt!
  timestamp: Instant!
  transactions: [Transaction!]! @cost(weight: 10.0)
}

type ContractCall implements ContractCallOrDeploy {
  transaction: Transaction! @cost(weight: 10.0)
  deploy: ContractDeploy! @cost(weight: 10.0)
  address: String!
  state: String!
  zswapChainState: String!
  operation: String!
}

type ContractDeploy implements ContractCallOrDeploy {
  transaction: Transaction! @cost(weight: 10.0)
  address: String!
  state: String!
  zswapChainState: String!
  definition: String!
}

type MerkleTreeCollapsedUpdate {
  update: String!
  start: BigInt!
  end: BigInt!
}

type Mutation {
  connect(viewingKey: String!): SessionId!
  disconnect(sessionId: SessionId!): Unit!
}

type ProgressUpdate {
  synced: BigInt!
  total: BigInt!
}

type Query {
  block(offset: BlockOffsetInput): Block
  transactions(hash: String, identifier: String): [Transaction!]!
  contract(address: String!, offset: BlockOffsetInput, transactionOffset: TransactionOffsetInput): ContractCallOrDeploy
}

type RelevantTransaction {
  transaction: Transaction!
  start: BigInt!
  end: BigInt!
}

type Subscription {
  transactions(offset: TransactionOffsetInput): TransactionSyncEvent
  wallet(sessionId: SessionId, index: BigInt, sendProgressUpdates: Boolean): WalletSyncEvent
  contract(address: String!, offset: BlockOffsetInput, transactionOffset: TransactionOffsetInput): ContractCallOrDeploy
  blocks(offset: BlockOffsetInput): Block
}

type Transaction {
  block: Block! @cost(weight: 10.0)
  hash: String!
  identifiers: [String!]!
  contractCalls: [ContractCallOrDeploy!]! @cost(weight: 10.0)
  raw: String!
  applyStage: String!
  merkleTreeRoot: String
}

type TransactionAdded {
  transaction: Transaction!
}

type ViewingUpdate {
  index: BigInt!
  update: [ZswapChainStateUpdate!]!
}